# Prepare Raspbian SD

## Propose:

Create your `Raspbian SD Card` from standard [raspberrypi.org](https://downloads.raspberrypi.org/) images
and do some custom configuration.

This tool is designed for people who have many Raspberry PIs and want to rebuild them with a
automatic process. But it can be used by anyone who wants to put a boot image on an SD card.

More details:

* Download image version if needed, then initialize SD.
* Customize SD (optional)
    * Set memory split,
    * Enable ssh
    * Add public keys on SD `/home/pi/.ssh/authorized_keys`,
    * Prepare [bcm runtime](https://git.cclaude.rocks/raspberry/bcm-runtime) configuration,
    * see [prepare-raspbian-sd-config.md](prepare-raspbian-sd-config.md) for more details.


`Raspbian` has a new name: `Raspberry Pi OS`, this version properly handle `Raspberry Pi OS` and `Raspbian`.


## Requirement:

* GNU Linux (fully tested under Mint)
* Some classic commands are required and will be installed on your system if they are missing:
    * `curl`
    * `jq`
    * `wget`


## How to clone this repository

Since this repository use **git sub-modules** you need to initialize and update embedded modules:

```bash
git clone --recurse-submodules https://git.cclaude.rocks/raspberry/prepare-raspbian-sd.git
```

Then follow installation section.


### How to update this repository

Since this repository use **git sub-modules** you need to initialize (if new sub-modules have been added)
and update embedded modules:

```bash
git fetch --verbose --prune # refresh git references
git pull --ff-only          # update content of current branch (if needed)
                            # don't merge with your modifications (if any)
git submodule init          # Initialize git sub-module (if needed)
git submodule update        # Update git sub-module (if needed)
git status                  # Show current status
```

Then follow installation section.

You can also just delete completely previous version and clone again

```bash
rm -frv prepare-raspbian-sd &&
git clone --recurse-submodules https://git.cclaude.rocks/raspberry/prepare-raspbian-sd.git
```


## Installation

When you have a copy of a version, or a clone run :

```bash
# cd PREPARE_RASPBIAN_SD_DIRECTORY
sudo bash prepare-raspbian-sd-install.sh
```

To remove the applications, use:

```bash
sudo bash /usr/local/share/prepare-raspbian-sd/prepare-raspbian-sd-remove.sh
```


### Install options

You can get help using :

```bash
bash ./prepare-raspbian-sd-install.sh --help
```

| Option                               | Description |
|--------------------------------------|-------------|
| -h, --help                           | Display help message and exit. |
| -S, --skip-update                    | Skip package manager update |
| -P PMEXE, --package-manager PMEXE    | Force package manager |


## Configuration

Optionally you can customize **Prepare Raspbian SD**.

You have two options to customize default download directory, witch is location where images will be store.

To be able to use `--configure` properly, you must choose how and what you want to configure.

See [prepare-raspbian-sd-config.md](prepare-raspbian-sd-config.md) for more details.


## Usage

To prevent mistakes the full process needed two steps:


### Initialize SD

For security reasons **SD card must be connected** to your computer but **not mounted**,
experimental parameter --auto-umount could be use with cautions.

According to parameters, it will download if needed last image, then it wil initialize SD card using this image.

```bash
prepare-raspbian-sd --put-last-image # To install last Raspbian image
```

or just:

```bash
prepare-raspbian-sd -L # To install last Raspbian image
```

**Warning**

New versions of `Raspbian` is are called `Raspberry Pi OS`, to get **new versions**,
you must use following commands:

```bash
prepare-raspbian-sd -L --arch armhf # 32 bits version (if your PI have 512 Mo, 1 Go, 2 Go or 4 GO of RAM)
prepare-raspbian-sd -L --arch arm64 # 64 bits version (if your PI have more than 4 GO of RAM)
```

If you want to use desktop version of `Raspberry Pi OS` (which is the default propose
on the [raspberrypi.org](https://raspberrypi.org/)) website, you should use:

```bash
prepare-raspbian-sd -L --full --arch armhf
```


### Customize SD

This step is optional: Customizing the SD card.

**SD card must be connected** to your computer **and mounted**.

What it could do:

* Enable **ssh** by creating a empty `ssh` file in root folder of boot partition.
* Set memory split to any value by adding related configuration to `config.txt` in root folder of boot partition.
* Customize **pi user** home directory:
    * Create folder `.bcm-workspace` for [bcm-runtime](https://git.cclaude.rocks/raspberry/bcm-runtime) project,
    * Create folder `.ssh` with secure configuration, and generate `.ssh/authorized_keys` if `SSH_AUTHORIZED_KEYS` is defined.

Then to customize SD. At the end of process SD card is sync and unmounted.

```bash
prepare-raspbian-sd --configure
```


### Initialize SD using custom URL

For instance to prepare an SD card based on [LibreElec](https://libreelec.tv/downloads/):

```bash
prepare-raspbian-sd --url http://releases.libreelec.tv/LibreELEC-RPi2.arm-8.2.5.img.gz --expected-img LibreELEC-RPi2.arm-8.2.5.img
```

Note that **LibreELEC** is supported. You can use `prepare-libreelec-sd` to initialize your
SD card with the last image ** LibreELEC ** using the command:

```bash
prepare-libreelec-sd
```

`prepare-libreelec-sd` is a wrapper for `prepare-raspbian-sd`.


### Download and unzip image using custom URL (SD card not required)

To download custom images, or manage a custom cache, you can use: **prepare-raspbian-sd-download-image**,
see [prepare-raspbian-sd-download-image.md](prepare-raspbian-sd-download-image.md) for documentation.


### Create image of your SD

To create an image of an existing SD card, use the following syntax:

```bash
prepare-raspbian-sd -T ~/my-sd-backup.img
```


## Related documentation

* [README.md](README.md),
* [prepare-raspbian-sd-config.md](prepare-raspbian-sd-config.md),
* [prepare-raspbian-sd-download-image.md](prepare-raspbian-sd-download-image.md).


## Related projects

* [bcm-runtime](https://git.cclaude.rocks/raspberry/bcm-runtime) project is design to manage your Linux (Raspbian, ...) hosts through ssh.

