#!/bin/bash
#
# bash-common-api:json-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
   # shellcheck disable=SC1091
  source "$( dirname "${BASH_SOURCE[0]}" )/message-api.sh" || exit $?
fi

function load_vars_from_json_file {
  local -r json_file="$1"
  local -r json_filter="$2"

  if [ ! -f "${json_file}" ] ; then
    echofatal "${FUNCNAME[0]}() - Can't find JSON file: '${json_file}'."
    return 100
  fi

  local json=
  json="$( jq -r "${json_filter}" < "${json_file}" )" || return $?
  local -r json

  if [ "${json}" = 'null' ] ; then
    echofatal "${FUNCNAME[0]}() - Can't find values for JSON filter: '${json_filter}'."
    return 100
  fi

  load_vars_from_json_string "${json}" || return $?
}

function load_vars_from_json_string {
  local -r json="$1"

  local keys=
  local key=

  if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
    keys="$( jq -r 'keys[] | select( contains( "@" ) )' <<<"${json}" )" || return $?

    while read -r key ; do
      if [ -n "${key}" ] ; then
        echodebug "Ignore '${key}'"
      fi
    done <<<"${keys}"
  fi

  # Typically ignore all keys like "VARNAME@DISABLED"
  keys="$( jq -r 'keys[] | select( contains( "@" ) | not )' <<<"${json}" )" || return $?
  local -r keys

  while read -r key ; do
    if [ -n "${key}" ] ; then
      __load_var_from_json_string__ || return $?
    fi
  done <<<"${keys}"
}

function __load_var_from_json_string__ {
  echoinfo_quiet "Discover '${key}'"

  local value_json=
  value_json="$( jq --arg key "${key}" '.[$key]' <<<"${json}" )" || return $?

  if [ "${value_json}" = 'null' ] ; then
    echoinfo_quiet "Value for '${key}' is null (ignore)"
    return 0
  fi

  local value=
  value="$( jq -r --arg key "${key}" '.[$key]' <<<"${json}" )" || return $?

  declare -grx "${key}"="$( envsubst <<<"${value}" )" || true # (x) required by envsubst (next iteration)
  if [ "${MESSAGE_API_SHOW_DEBUG:-}" = 'true' ] ; then
    echodebug "Set '${key}'='${!key}'"
  fi
}

function load_vars_from_json { # DEPRECATED 2021-10-04
  local -r json_file="$1"
  local -r json_path="$2"

  echowarn "${FUNCNAME[0]}() is deprecated, use load_vars_from_json_file() instead"

  if [ ! -f "${json_file}" ] ; then
    echofatal "${FUNCNAME[0]}() - Can't find JSON file: '${json_file}'."
    return 1
  fi

  local keys=
  keys="$( jq -re "${json_path} | keys[]" < "${json_file}" )" || return $?

  local key=
  local value=
  while read -r key ; do
    if [ -n "${key}" ] ; then
      echoinfo "Set '${key}'"
      value="$( jq -re "${json_path}.${key}" < "${json_file}" )" || return $?

      declare -gr "${key}"="${value}"
      # shellcheck disable=SC2163
      export "${key}" # required by envsubst
    fi
  done <<<"${keys}"
}

function get_expanded_value_from_json {
  local -r json_str="$1"
  local -r key="$2"

  local value=
  value="$( jq -re "${key}" <<<"${json_str}" )" || return $?

  envsubst <<<"${value}" || return $?
}

function get_optional_expanded_value_from_json {
  local -r json_str="$1"
  local -r key="$2"

  local value=
  value="$( jq -r "${key}" <<<"${json_str}" )" || return $?

  envsubst <<<"${value}" || return $?
}

function is_null_or_empty {
  local -r value="$1"

  if [ "${value}" = 'null' ] ; then
    return 0
  elif [ -z "${value}" ] ; then
    return 0
  fi

  return 1
}
