#!/bin/bash
#
# prepare-raspbian-sd:bin/api/prepare-raspbian-detect-sd-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 100
fi

function detect_sd_get_SD_device {
  local dev_path=
  for dev_path in /sys/block/* ; do
    if [ -f "${dev_path}/device/type" ] ; then
      if [ "$( cat "${dev_path}/device/type" )" = 'SD' ] ; then
        basename "${dev_path}"
      fi
     fi
  done
}

function __raspios_check_boot_ui_type__ {
  local -r boot_home="$1"
  local -r issue_file="${boot_home}/issue.txt"

  if [ ! -f "${issue_file}" ] ; then
    echo 'NOT-RASPI'
    echofatal "Can't find: '${issue_file}'"
    return 100
  fi

  local __gui_text__=
  __gui_text__="$( grep 'Generated using pi-gen' "${issue_file}" | cut -d',' -f4 | sed -E 's,[[:blank:]],,' )"

  case "${__gui_text__}" in
    'stage2')
      echo 'ligth'
      ;;

    'stage4')
      echo 'desktop'
      ;;

    *)
      echoerror "Can't find text/gui mode: '${__gui_text__}'"
      echo 'UNKNOWN-TYPE'
      ;;
  esac
}

function __raspios_check_boot_arch__ {
  local -r boot_home="$1"
  local -r config_file="${boot_home}/config.txt"

  if [ ! -f "${config_file}" ] ; then
    echo 'NOT-RASPI'
    echofatal "Can't find: '${config_file}'"
    return 100
  fi

  local __arch__=
  __arch__="$( grep '^arm_64bit=' "${config_file}" | sed -E -e 's,(.*)=(.*),\2,' -e 's,[[:blank:]],,' )"

  case "${__arch__}" in
    '1')
      echo 'arm64'
      ;;

    ''|'0')
      echowarn "arch: '${__arch__}'"
      echo 'armhf'
      ;;

    *)
      echoerror "Can't find arch: '${__arch__}'"
      echo 'UNKNOWN-ARCH'
      ;;
  esac
}

function detect_sd_display_boot_part_type {
  local -r boot_home="$1"

  local __ui_type__=
  __ui_type__="$( detect_sd_get_ui_type "${boot_home}" )" || return $?
  local -r __ui_type__

  local __arch_type__=
  __arch_type__="$( __raspios_check_boot_arch__ "${boot_home}" )" || return $?
  local -r __arch_type__

  echoinfo "SD boot partition located '${boot_home}' is for CPU '${__arch_type__}' using '${__ui_type__}' version"
}

function detect_sd_get_arch {
  local -r boot_home="$1"

  __raspios_check_boot_arch__ "${boot_home}" || return $?
}

function detect_sd_get_ui_type {
  local -r boot_home="$1"

  __raspios_check_boot_ui_type__ "${boot_home}" || return $?
}

