#!/bin/bash
#
# prepare-raspbian-sd:bin/api/raspberrypi-org-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 100
fi

# return: 'armhf' or 'arm64'
function raspberrypi_org_get_valid_arch_for_url {
  case "${arch_type}" in
    'armhf'|'arm32'|''|'armv7l')
       echo 'armhf'
       ;;

    'arm64'|'aarch64')
       echo 'arm64'
       ;;

    *)
      echofatal "${FUNCNAME[0]}() - Unsupported arch type: '${arch_type}'"
      return 100
      ;;
  esac
}

function raspberrypi_org_get_url_raspbian_last_version { # was: build_last_version_url_for_raspbian
  local -r os_type="$1"

  local lite=
  if [ "${os_type}" = 'lite' ] ; then
    lite='lite_'
  fi

  local -r url="https://downloads.raspberrypi.org/raspbian_${lite}latest"

  echoinfo "Loading version from: ${url}" >&2

  curl --head "${url}" |
    grep -i '^Location: ' |
    cut -d ' ' -f2- |
    tr '\r' '\n' |
    head -n 1
}

function raspberrypi_org_get_url_raspios_last_version { # was: build_last_version_url_for_raspberry_pi_os
  local -r arch_type="$1"
  local -r os_type="$2"

  local identifier=
  case "${os_type}" in
    'lite')
      identifier="lite_${arch_type}_latest"
      ;;

    'full')
      identifier="${arch_type}_latest"
      ;;
  esac
  local -r url="https://downloads.raspberrypi.org/raspios_${identifier}"

  echoinfo "Loading version from: ${url}" >&2

  curl --head "${url}" |
    grep -i '^Location: ' |
    cut -d ' ' -f2- |
    tr '\r' '\n' |
    head -n 1
}

#
# Sample:
#   raspberrypi_org_get_json 'buster' 'arm64' 'full'
#
function raspberrypi_org_get_json { # raspios_download_get_json
  local -r os_name="$1"     # 'buster' or 'bullseye'
  local -r arch_type="$2"   # 'armhf', 'arm32', '', 'armv7l' or 'arm64', 'aarch64'
  local -r os_type="$3"     # 'lite' or 'full'

  local arch_type_for_url= # 'armhf' or 'arm64'
  arch_type_for_url="$( raspberrypi_org_get_valid_arch_for_url "${arch_type}" )" || return $?
  local -r arch_type_for_url # 'armhf' or 'arm64'

  local identifier=
  case "${os_type}" in
    'lite')
      identifier="-${arch_type_for_url}-lite.img"
      ;;

    'full')
      identifier="${arch_type_for_url}.img"
      ;;

    *)
      echofatal "${FUNCNAME[0]}() - Unsupported os type: '${os_type}'"
      return 100
      ;;
  esac

  local remote_json=
  remote_json="$( curl --silent --location 'https://downloads.raspberrypi.org/os_list_imagingutility_v3.json' )" || return $?

  local filtered_json=
  filtered_json="$(
    set -x
    jq <<<"${remote_json}" \
      --arg osname "-${os_name}-" \
       --arg identifier "${identifier}" \
       '.os_list[] | .subitems // [] | .[] | select( (.url // empty) | contains( $osname ) ) | select( .url | contains( $identifier ) )'
  )" || return $?

  if [ -z "${filtered_json}" ] ; then
    echofatal "Can't find entry for with osname='${os_name}' and  identifier='${identifier}'"

    echoerror "Results with os_name: '${os_name}'"
    grep --color -i "${os_name}" <<<"${remote_json}" >&2

    echoerror "Results with arch: '${arch_type_for_url}'"
    grep --color -i "${arch_type_for_url}" <<<"${remote_json}" >&2

    return 100
  fi
  echo "${filtered_json}"
}

#
# cat 2022-05-01.os_list_v3.json | jq '.os_list[] | select( .version == "stretch" )' # bullseye
#

