#!/bin/bash
#
# prepare-raspbian-sd:bin/api/raspios-download-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 100
fi

function __raspios_download_init_variables_from_html__ { # was: init_raspios_raspbian_values
  local -r arch_type="$1"
  local -r os_type="$2" # 'full' or 'lite'

  if [ -z "${arch_type}" ] ; then
    LAST_VERSION_URL="$( raspberrypi_org_get_url_raspbian_last_version "${os_type}" )"
  else
    LAST_VERSION_URL="$( raspberrypi_org_get_url_raspios_last_version "${arch_type}" "${os_type}" )"
  fi
  LAST_VERSION_FILENAME="$( basename "${LAST_VERSION_URL}" )"
}

#
# __raspios_download_init_variables_from_json__ "${os_name}" "${arch}" "${os_type}"
#
function __raspios_download_init_variables_from_json__ { # was: init_raspios_raspbian_values_from_json
  local -r os_name="$1"
  local -r arch_type="$2"
  local -r os_type="$3" # 'full' or 'lite'

  local _json_=
  _json_="$( raspberrypi_org_get_json "${os_name}" "${arch_type}" "${os_type}" )" || return $?
  local -r _json_

  LAST_VERSION_URL="$( jq -r '.url' <<<"${_json_}" )" || return $?
  LAST_VERSION_FILENAME="$( basename "${LAST_VERSION_URL}" )"
}

function raspios_download_init_variables {
  local -r os_name="$1"     # '', 'buster' or 'bullseye'
  local -r arch_type="$2"   #
  local -r os_type="$3"     # 'full' or 'lite'
  local -r download_image_directory="$4"

  local arch_type_for_url= # 'armhf' or 'arm64'
  arch_type_for_url="$( raspberrypi_org_get_valid_arch_for_url "${arch_type}" )" || return $?
  local -r arch_type_for_url # 'armhf' or 'arm64'

  if [ -z "${os_name}" ] ; then
    __raspios_download_init_variables_from_html__ "${arch_type_for_url}" "${os_type}" || return $?
  else
    __raspios_download_init_variables_from_json__ "${os_name}" "${arch_type_for_url}" "${os_type}" || return $?
  fi

  if [ -z "${LAST_VERSION_URL}" ] ; then
    echofatal 'Cannot get last version - Check your Internet connection'
    exit 100
  fi

  EXPECTED_ZIP_FILE="${download_image_directory}/${LAST_VERSION_FILENAME}"

  # Remove archive extension
  EXPECTED_IMG_FILE="$( sed -E 's,^(.*)(.xz|.zip|.gz),\1,g' <<<"${EXPECTED_ZIP_FILE}" )"
  # Ensure file ending with .img
  EXPECTED_IMG_FILE="${EXPECTED_IMG_FILE%.img}.img"

  DOWNLOAD_SHA256_URL="${LAST_VERSION_URL}.sha256"
  DOWNLOAD_SHA256_FILE="${EXPECTED_ZIP_FILE}.sha256"

  declare -grx DOWNLOAD_SHA256_FILE
  declare -grx DOWNLOAD_SHA256_URL
  declare -grx EXPECTED_IMG_FILE
  declare -grx EXPECTED_ZIP_FILE
  declare -grx LAST_VERSION_FILENAME
  declare -grx LAST_VERSION_URL

  raspios_download_show_variables
}

function raspios_download_show_variables {
  cat <<EOF >&2
* DOWNLOAD_SHA256_FILE    : '${DOWNLOAD_SHA256_FILE}'
* DOWNLOAD_SHA256_URL     : '${DOWNLOAD_SHA256_URL}'
* EXPECTED_IMG_FILE       : '${EXPECTED_IMG_FILE}'
* EXPECTED_ZIP_FILE       : '${EXPECTED_ZIP_FILE}'
* LAST_VERSION_FILENAME   : '${LAST_VERSION_FILENAME}'
* LAST_VERSION_URL        : '${LAST_VERSION_URL}'
EOF
}
