# prepare-raspbian-sd-config

Allow to configure `prepare-raspbian-sd`.

## Usage

```
   prepare-raspbian-sd-config --help
   prepare-raspbian-sd-config --get VARNAME
   prepare-raspbian-sd-config --list
   prepare-raspbian-sd-config --list-all
   prepare-raspbian-sd-config --set-boolean VARNAME VALUE
   prepare-raspbian-sd-config --set-integer VARNAME VALUE
   prepare-raspbian-sd-config --set-link VARNAME LINK_TARGET
   prepare-raspbian-sd-config [--set] VARNAME VALUE
```

### Options

```
  -h, --help                Display help message and exit.
      --get VARNAME         Show given variable content
  -l, --list                Display valid variables name's
  -L, --list-all            Show all define variables in JSON format.
      --set-boolean VARNAME VALUE
                            Set given VALUE to variable VARNAME. VALUE must be a boolean (true|false).
      --set-integer VARNAME VALUE
                            Set given VALUE to variable VARNAME. VALUE must be an integer.
      --set-link VARNAME LINK_TARGET
                            The value of the VARNAME field corresponds to the content of the file defined
                            by LINK_TARGET.
                            The associated value will be the raw content of this file.
      --set VARNAME VALUE   Set given VALUE to variable VARNAME (String).
```

## Configuration sample

You have two options to customize default download directory, witch is location where images will be store.
First you can use `prepare-raspbian-sd-config`:

```bash
_DOWNLOAD_="$( xdg-user-dir DOCUMENTS )/raspbian-images/download"
_BACKUP_="$( xdg-user-dir DOCUMENTS )/raspbian-images/backups"

# Create directories if needed
mkdir -vp "${_DOWNLOAD_}" "${_BACKUP_}"

# Configuration (recommended)
prepare-raspbian-sd-config RASPBIAN_DEFAULT_BACKUP_DIRECTORY "${_BACKUP_}"
prepare-raspbian-sd-config RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY "${_DOWNLOAD_}"
prepare-raspbian-sd-config SSH_AUTHORIZED_KEYS "$( cat ~/.ssh/id_rsa.pub )"
prepare-raspbian-sd-config --set-boolean BCM_WORKSPACE true
prepare-raspbian-sd-config --set-boolean SSH_ENABLE true
prepare-raspbian-sd-config --set-boolean SSH_CONFIGURE true

# Configuration (optional - if used ensure value is what you want)
prepare-raspbian-sd-config DEFAULT_SD_DEV_NAME 'mmcblk0'
prepare-raspbian-sd-config GPU_MEM '16'
prepare-raspbian-sd-config RASPBIAN_DEFAULT_IMG "${_DOWNLOAD_}/2019-07-10-raspbian-buster-lite.img"
```

Or you can export required variables (using `~/.bashrc` or custom script):

```bash
export RASPBIAN_SSH_CONFIG='/path/to/your-ssh-need.config'
```

* If `RASPBIAN_DEFAULT_BACKUP_DIRECTORY` not define, commands your `Documents` directory.
* If `RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY` not define, commands use your `Download` directory.


## Variables description

| Variable name                               | Usage description                         | Typical values (type) |
|---------------------------------------------|-------------------------------------------|-----------------------|
| `BCM_WORKSPACE`                             | Optional, defaults to true. If set to false, the `/home/pi/.bcm-workspace` folder will not be created. | `true` (boolean) |
| `DEFAULT_SD_DEV_NAME`                       | Optional, use to write on SD card.        | `mmcblk0` (string) |
| `GPU_MEM`                                   | Optional, use by `--configure`. Configure memory split, if not set or if set to 0 memory split configuration is skipped | `16` (integer) |
| `RASPBIAN_DEFAULT_BACKUP_DIRECTORY`         | Required to be able to use `--full-clone`. Variable must contain full path of an existing directory where your SD backups will be stored. | `/home/username/Documents/` (string) |
| `RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY` | Nice to have to use `--download` option of **prepare-raspbian-sd-download-image** and to use `--put-last-image` option of **prepare-raspbian-sd**. | `/home/username/Documents/raspbian-images` (string) |
| `RASPBIAN_DEFAULT_IMG`                      | Required to be able to use `--put-default-image`. Variable must contain full path to your `raspbian-stretch-lite.img` local file. | `/path/to/2018-10-09-raspbian-stretch-lite.img` (string) |
| `RASPBIAN_PI_GROUP_ID`                      | Optional, default value 1000              | `1000` (integer) |
| `RASPBIAN_PI_USER_ID`                       | Optional, default value 1000              | `1000` (integer) |
| `SSH_AUTHORIZED_KEYS`                       | Required to be able to use `--configure`. Variable must contain content of `/home/pi/.ssh/authorized_keys` you want to add. | `"$( cat ~/.ssh/id_rsa.pub )"` (string) |
| `SSH_CONFIGURE`                             | Optional, if set to true create `~/.ssh directory` | `true` (boolean) |
| `SSH_ENABLE`                                | Optional, if set to true enable sshd.     | `true` (boolean) |


## Related documentation

* [README.md](README.md),
* [prepare-raspbian-sd-config.md](prepare-raspbian-sd-config.md),
* [prepare-raspbian-sd-download-image.md](prepare-raspbian-sd-download-image.md).

