#!/usr/bin/env bats
#
# prepare-raspbian-sd:tests/test.bats
#
setup() {
  # get the containing directory of this file
  # use $BATS_TEST_FILENAME instead of ${BASH_SOURCE[0]} or $0,
  # as those will point to the bats executable's location or the preprocessed file respectively
  PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BATS_TEST_FILENAME}" )" )" )"

  # make executables in src/ visible to PATH
  # PATH="$DIR/../src:$PATH"

  declare -gr EXE_PREPARE_RASPBIAN_SD_CONFIG="${PROJECT_HOME}/bin/prepare-raspbian-sd-config.sh"
  declare -gr HOME_SAVED="${HOME}"
  declare -gr HOME_READONLY="${PROJECT_HOME}/tests/data/HOME.readonly"
  declare -gr HOME_READWRITE="${PROJECT_HOME}/.tmp/HOME.readwrite"

  export MESSAGE_API_HIDE_DATE='true'
}

@test "prepare-raspbian-sd-config --help" {
  HOME="${HOME_SAVED}"

  run "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --help
  [ "$status" -eq 0 ]
# echo "# output=${output}" >&3
}

@test "prepare-raspbian-sd-config --show-filename" {
  HOME="${HOME_READONLY}"

# echo "# HOME=${HOME}" >&3
  run "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --show-filename
  [ "$status" -eq 0 ]
# echo "# output=${output}" >&3
  [ "$output" = "${HOME}/.config/prepare-raspbian-sd/config.json" ]
}

@test "prepare-raspbian-sd-config --list-all" {
  HOME="${HOME_READONLY}"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --list-all
  [ "$status" -eq 0 ]
  run jq -c '.' <<<"${output}"
  [ "$status" -eq 0 ]
# echo "# output=${output}" >&3
  [ "${#lines[@]}" -eq 1 ]
}

@test "prepare-raspbian-sd-config --list-all (no config)" {
  HOME="${HOME_READWRITE}00"

  file="$( "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --show-filename )"
  [ -f "${file}" ] && rm "${file}"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --list-all
  echo "# status=${status}" >&3
  echo "# stderr=${stderr}" >&3
# echo "# output=${output}" >&3
  [ "$status" -eq 0 ]

  run jq -c -S '.' <<<"${output}"
# echo "# output=${output}" >&3
  [ "${status}" -eq 0 ]
  [ "${output}" = '{"BCM_WORKSPACE":null,"DEFAULT_SD_DEV_NAME":null,"GPU_MEM":null,"RASPBIAN_DEFAULT_BACKUP_DIRECTORY":null,"RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY":null,"RASPBIAN_DEFAULT_IMG":null,"RASPBIAN_PI_GROUP_ID":null,"RASPBIAN_PI_USER_ID":null,"SSH_AUTHORIZED_KEYS":null,"SSH_CONFIGURE":null,"SSH_ENABLE":null}' ]
  [ "${#lines[@]}" -eq 1 ]
}

@test "prepare-raspbian-sd-config --list" {
  HOME="${HOME_READONLY}"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --list
  [ "$status" -eq 0 ]
  result="${output}"
# echo "# output=${output}" >&3

  run jq -r '.[]' <<<"${result}"
  [ "$status" -eq 0 ]
# echo "# output=${output}" >&3
  [ "${lines[0]}" = 'BCM_WORKSPACE' ]
  [ "${lines[1]}" = 'DEFAULT_SD_DEV_NAME' ]
  [ "${lines[2]}" = 'GPU_MEM' ]
  # ...
  [ "${lines[10]}" = 'SSH_ENABLE' ]
  [ "${#lines[@]}" -eq 11 ]
}

@test "prepare-raspbian-sd-config --get 'BCM_WORKSPACE' # boolean" {
  HOME="${HOME_READONLY}"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --get 'BCM_WORKSPACE'
  [ "$status" -eq 0 ]
  result="${output}"

  run jq -r 'to_entries[].value' <<<"${result}"
  [ "$status" -eq 0 ]
# echo "# output=${output}" >&3
  [ "${lines[0]}" = 'true' ]
  [ "${#lines[@]}" -eq 1 ]
}

@test "prepare-raspbian-sd-config --get FaTaL" {
  HOME="${HOME_READONLY}"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --get 'FaTaL'
  [ "$status" -eq 11 ]
  [ -z "$output" ]
}

@test "prepare-raspbian-sd-config --get 'SSH_AUTHORIZED_KEYS' # link" {
  HOME="${HOME_READONLY}"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --get 'SSH_AUTHORIZED_KEYS'
# echo "# status=${status}" >&3
# echo "# stderr=${stderr}" >&3
# echo "# output=${output}" >&3
  [ "$status" -eq 0 ]
  result="${output}"

  run jq -r 'to_entries[].value' <<<"${result}"
  [ "$status" -eq 0 ]
# echo "# output=${output}" >&3
  [ "${lines[0]}" = 'SSH_AUTHORIZED_KEYS_content' ]
  [ "${#lines[@]}" -eq 1 ]
}

@test "prepare-raspbian-sd-config.sh --set-boolean FaTaL VALUE" {
  HOME="${HOME_READWRITE}01"

  file="$( "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --show-filename )"
  [ -f "${file}" ] && rm "${file}"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --set-boolean 'FaTaL' 'VALUE'
  echo "# status=${status}" >&3
  echo "# stderr=${stderr}" >&3
# echo "# output=${output}" >&3
  [ "$status" -eq 11 ]
  [ -z "$output" ]
}

@test "prepare-raspbian-sd-config.sh --set-boolean BCM_WORKSPACE VALUE" {
  HOME="${HOME_READWRITE}02"

  file="$( "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --show-filename )"
  [ -f "${file}" ] && rm "${file}"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --set-boolean BCM_WORKSPACE VALUE
  echo "# stderr=${stderr}" >&3
  echo "# status=${status}" >&3
  echo "# output=${output}" >&3
  [ "$status" -eq 22 ]
  [ -z "$output" ]
}

@test "prepare-raspbian-sd-config.sh --set-boolean BCM_WORKSPACE true" {
  HOME="${HOME_READWRITE}02"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --set-boolean BCM_WORKSPACE true
# echo "# output=${output}" >&3
  [ "$status" -eq 0 ]
  [ -z "$output" ]
}

@test "prepare-raspbian-sd-config.sh --set-boolean BCM_WORKSPACE false" {
  HOME="${HOME_READWRITE}02"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --set-boolean BCM_WORKSPACE false
# echo "# output=${output}" >&3
  [ "$status" -eq 0 ]
  [ -z "$output" ]
}

@test "prepare-raspbian-sd-config.sh --set-integer BCM_WORKSPACE VALUE # FAIL" {
  HOME="${HOME_READWRITE}02"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --set-integer BCM_WORKSPACE VALUE
  echo "# stderr=${stderr}" >&3
  echo "# status=${status}" >&3
  echo "# output=${output}" >&3
  [ "$status" -eq 21 ]
  [ -z "$output" ]
}

@test "prepare-raspbian-sd-config.sh --set-integer RASPBIAN_PI_GROUP_ID NAN" {
  HOME="${HOME_READWRITE}02"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --set-integer RASPBIAN_PI_GROUP_ID NAN
  echo "# stderr=${stderr}" >&3
  echo "# status=${status}" >&3
# echo "# output=${output}" >&3
  [ "$status" -eq 23 ]
  [ -z "$output" ]
}

@test "prepare-raspbian-sd-config.sh --set-integer RASPBIAN_PI_GROUP_ID 1024" {
  HOME="${HOME_READWRITE}02"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --set-integer RASPBIAN_PI_GROUP_ID 1024
  echo "# stderr=${stderr}" >&3
  echo "# status=${status}" >&3
  echo "# output=${output}" >&3
  [ "$status" -eq 0 ]
  [ -z "$output" ]
}

@test "prepare-raspbian-sd-config.sh --set-link SSH_AUTHORIZED_KEYS NoTaFiLe # FAIL" {
  HOME="${HOME_READWRITE}02"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --set-link SSH_AUTHORIZED_KEYS NoTaFiLe
  echo "# stderr=${stderr}" >&3
  echo "# status=${status}" >&3
  echo "# output=${output}" >&3
  [ "$status" -eq 24 ]
  [ -z "$output" ]
}

@test "prepare-raspbian-sd-config.sh --set-link SSH_AUTHORIZED_KEYS README" {
  HOME="${HOME_READWRITE}02"
  link_file="${HOME_READONLY}/SSH_AUTHORIZED_KEYS"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --set-link SSH_AUTHORIZED_KEYS "${link_file}"
  echo "# stderr=${stderr}" >&3
  echo "# status=${status}" >&3
  echo "# output=${output}" >&3
  [ "$status" -eq 0 ]
  [ -z "$output" ]
}

#  prepare-raspbian-sd-config.sh --set-link VARNAME LINK_TARGET # TODO need to check var in filename (not supported (yet?))

#  prepare-raspbian-sd-config.sh [--set] VARNAME VALUE

@test "prepare-raspbian-sd-config.sh --set RASPBIAN_DEFAULT_BACKUP_DIRECTORY VALUE " {
  HOME="${HOME_READWRITE}02"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" --set RASPBIAN_DEFAULT_BACKUP_DIRECTORY VALUE
  echo "# stderr=${stderr}" >&3
  echo "# status=${status}" >&3
  echo "# output=${output}" >&3
  [ "$status" -eq 0 ]
  [ -z "$output" ]
}

@test "prepare-raspbian-sd-config.sh RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY VALUE " {
  HOME="${HOME_READWRITE}02"

  run --separate-stderr "${EXE_PREPARE_RASPBIAN_SD_CONFIG}" RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY VALUE
  echo "# stderr=${stderr}" >&3
  echo "# status=${status}" >&3
  echo "# output=${output}" >&3
  [ "$status" -eq 0 ]
  [ -z "$output" ]
}

@test "addition using bc" {
  result="$(echo 2+2 | bc)"
  [ "$result" -eq 4 ]
}
